c*********************************************************************
c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/cplmean.F,v $
c $Author: munnich $
c $Revision: 1.3 $
c $Date: 2000/05/29 19:36:37 $
c
c File: cplmean.f
c
c Cplmean accumulates coupling variables (fluxes) from the qtcm over
c time.  The qtcm time step is supplied as the argument "it".  If
c "it" equals 1, cplmean clears the flux buffers before accumulating.
c The fluxes are returned as time means by the routines 
c GetStress and GetSfcHeat.
c
c Code History:
c   Original version:  Bill Weibel  Spring 1997

c*********************************************************************

      Subroutine CplMean(it)
c
c Accumulate coupling variables over the length of the coupling time step
c 
      implicit none
#include "hgrid.h"
#include "qtcm.h"

c Arguments:
      integer it	! qtcm time step

c Commons
      REAL samples, FLWdscum(Nx,Ny),FLWuscum(Nx,Ny)
      REAL FSWdscum(Nx,Ny),FSWuscum(Nx,Ny)
      REAL Evapcum(Nx,Ny),FTscum(Nx,Ny)
      common /cplcount/ samples
      common /SfcHeat/ FSWdscum,FSWuscum,FLWdscum,FLWuscum,
     $                 Evapcum,FTscum
      REAL tauxcum(NX,NY),tauycum(NX,NY)
      REAL Qccum(NX,NY)
      common /SfcStress/  tauxcum,tauycum
      common /WaterFlux/  Qccum
      save /cplcount/, /SfcHeat/, /SfcStress/

c Local variables
      integer i,j

c Sum the surface fluxes
c   
      if (it .eq. 1) then
         call as(FSWdscum,Nx*Ny,zero)
         call as(FSWuscum,Nx*Ny,zero)
         call as(FLWdscum,Nx*Ny,zero)
         call as(FLWuscum,Nx*Ny,zero)
         call as(Evapcum,Nx*Ny,zero)
         call as(FTscum,Nx*Ny,zero)
         call as(tauxcum,Nx*Ny,zero)
         call as(tauycum,Nx*Ny,zero)
         call as(Qccum,Nx*Ny,zero)
         samples = 0.0
      end if

      samples = samples + 1.0
      do j = 1,Ny
         do i=1,Nx
            FSWdscum(i,j) = FSWdscum(i,j) + FSWds(i,j)
            FSWuscum(i,j) = FSWuscum(i,j) + FSWus(i,j)
            FLWdscum(i,j) = FLWdscum(i,j) + FLWds(i,j)
            FLWuscum(i,j) = FLWuscum(i,j) + FLWus(i,j)
            Evapcum(i,j) = Evapcum(i,j) + Evap(i,j)
            FTscum(i,j) = FTscum(i,j) + FTs(i,j)
            tauxcum(i,j) = tauxcum(i,j) + taux(i,j)
            tauycum(i,j) = tauycum(i,j) + tauy(i,j)
            Qccum(i,j) = Qccum(i,j) + Qc(i,j)
         end do
      end do

      return
      end


      Subroutine getStress(taux_out,tauy_out)
c
c Coupling interface routine.
c Return time-averaged wind stress up through argument list
c
      implicit none
#include "hgrid.h"

c --Local Variables:
      REAL taux_out(NX,NY),tauy_out(NX,NY)
      integer i,j
c -- Globals
      REAL divisor, samples
      common /cplcount/ samples
      REAL tauxcum(NX,NY),tauycum(NX,NY)
      common /SfcStress/  tauxcum,tauycum
c*****************************************************************

      divisor = 1.0/samples
      do j=1,NY
         do i=1,NX
            taux_out(i,j) = tauxcum(i,j) * divisor
            tauy_out(i,j) = tauycum(i,j) * divisor
         end do
      end do
      
      return
      end
 
      Subroutine getPrec(Qc)
c
c Coupling interface routine.
c Return time-averaged precipitation up through argument list
c Evaporation can be obtained via getSfcHeat
c NOTE: Unit are energy unit
c
      implicit none
#include "hgrid.h"

c --Local Variables:
      REAL Qc(NX,NY)
      integer i,j
c -- Globals
      REAL divisor, samples
      common /cplcount/ samples
      REAL Qccum(NX,NY)
      common /WaterFlux/  Qccum
c*****************************************************************

      divisor = 1.0/samples
      do j=1,NY
         do i=1,NX
            Qc(i,j) = Qccum(i,j) * divisor
         end do
      end do
      
      return
      end

      Subroutine getSfcHeat(FSWds,FSWus,FLWds,FLWus,Evap,FTs)
c
c GetSfcHeat is an interface for Ocean-Atmosphere coupling.
c There are no inputs.  The fluxes, appearing as arguments, are synonymous
c with variables defined in qtcm.h.  But, in this case, they are averages
c over some number of time steps.  The number of steps is given by 'samples'
c in the common block 'SfcHeat'
c
c Bugs:
c     The driver calls Ocean before QTCM.  So this routine 
c     may be called before the flux variables are defined.
c
      implicit none
#include "hgrid.h"

      integer i,j
      REAL divisor, samples
      REAL FLWdscum(Nx,Ny),FLWuscum(Nx,Ny)
      REAL FSWdscum(Nx,Ny),FSWuscum(Nx,Ny)
      REAL Evapcum(Nx,Ny),FTscum(Nx,Ny)
      common /cplcount/ samples
      common /SfcHeat/ FSWdscum,FSWuscum,FLWdscum,FLWuscum,
     $                 Evapcum,FTscum
      REAL FSWds(Nx,Ny),FSWus(Nx,Ny)
      REAL FLWds(Nx,Ny),FLWus(Nx,Ny)
      REAL Evap(Nx,Ny),FTs(Nx,Ny)
      data samples/1.0/			! avoid divide by zero

      divisor = 1.0/samples
      do j=1,Ny
         do i=1,Nx
            FSWds(i,j) = FSWdscum(i,j) * divisor
            FSWus(i,j) = FSWuscum(i,j) * divisor
            FLWds(i,j) = FLWdscum(i,j) * divisor
            FLWus(i,j) = FLWuscum(i,j) * divisor
            Evap(i,j) = Evapcum(i,j) * divisor
            FTs(i,j) = FTscum(i,j) * divisor
         end do
      end do
      return
      end
