c*********************************************************************
c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/mxlayer.F,v $
c $Author: munnich $
c $Revision: 1.3 $
c $Date: 2000/09/09 22:00:38 $
c
c File: mxlayer.f
c
c Code history:
c   Original version:  H. Su/N. Zeng Nov. 1998
c
c   *       in driver.f
c*********************************************************************


      Subroutine OceanInit()
c
c  Get the initial SST data for the dates on either side of an
c  arbitrary starting date
c
      implicit none
      include 'hgrid.h'
      include 'driver.h'
      include 'mxlayer.h'
      include 'calendar.h'
      
      integer i,j
c     integer lnblnk          !Function
  
      write (*,*) "OceanInit: get the initial SST data"

c get init SST using bndry1 which does interpolation
      call bndry1('sst',SSTdir,Tnow)

c read the initial SST
c     open(13,file=SSTdir(1:lnblnk(SSTdir))//'/00000000.sst'
c    &       ,form='formatted',status='old')
c     read(13,*) Tnow                                  !in Celsius
c     close(13)

      return
      end
      
      
      Subroutine Ocean(ndays,it)
c*********
c  compute SST using mxlayer model
      implicit none
      include 'hgrid.h'
      include 'qtcm.h'
      integer ndays,it
c***************************************************************
    
      call getQflux           !get Qflux

      call getSfcHeat(FSWds,FSWus,FLWds,FLWus,Evap,FTs)   !get averaged sfc 
                     !fluxes accumulated over previous coupled interval 
                     ! temporarily stored in FSWds etc.
                     !  (e.g. 1day); [cplmean] has been called in [qtcm]

      call mxstep(ndays)      !time-stepping of the mix-layer ocean

      return
      end
      
      subroutine getQflux
c*********
c  read in Qflux
c
      include 'hgrid.h'
      include 'driver.h'
      include 'mxlayer.h'
      logical firstcall
      data firstcall/.true./
      save firstcall
c******************************************************
c     
c  get Q fluxes 
c
      if(SSTmode.eq.'perpetual') then     ! constant Qflux such as Annual Mean
         if(firstcall) then
            open(16,file='00000000.fsn', status='old')
            do j=1,NY
            do i=1,NX
               read(16,*) qfx(i,j)       
            enddo
            enddo
            firstcall=.false.
         endif
      elseif(SSTmode.eq.'seasonal') then   !seasonally varying Qflux; Fs and Ts 
         call bndry1('fsn','./',fsn)       !  tendency are treated differently
         call bndry2('dts','./',dts)
         do j=1,NY
         do i=1,NX
            qfx(i,j)=fsn(i,j)-dts(i,j)
         enddo
         enddo
      endif 

      return
      end

      subroutine getQfa(qfa)
c**********************************
!
! read qflux anomalies from binary file "uom_qflx_an_q.seq"
      implicit none
      include 'hgrid.h'
#include "calendar.h"
      
      REAL qfa(NXY)
      integer ncall, monthread, i
      data ncall,  monthread /0, 0/
      save ncall, monthread
      
      if(ncall.eq.0)
     $     open(19,file='../bnddata/qflux_anomalies/uom_qflx_an_q.seq',
     $        form='unformatted', access='sequential')


      if(monthread.ne.monthofyear) then
            read(19) qfa
            ncall=ncall+1
            monthread=monthofyear
            print*,'getQfa: Read record',ncall,
     $           ' of qflux anomalies at date ',dateofmodel
            call flush(6)
            do i=1,nxy
               if(qfa(i).ge.10000) qfa(i)=0.0
            enddo
      endif
      
      return
      end

#endif
      
      
      

      subroutine mxstep(intcpl)
c
      implicit none
      include 'hgrid.h'
      include 'driver.h'
      include 'qtcm.h'
      include 'mxlayer.h'

c --Altered Common Variables:
c Tnow
 
c --Local Variables & Functions:
      REAL Rsnet,Fsnet,Dmx,Cmx
      integer i,j,intcpl

      data Dmx/50./     !Depth of the mix-layer ocean;  meters
      save Dmx

c*****************************************************************
c  water heat capacity * density * Depth
      Cmx=4.18e6*Dmx      !1cal/g/K * 1g/cm3 * Depth;  in J/K/M2
c******************************************************************
      
      do j=1,NY
      do i=1,NX

       if(STYPE(i,j).eq.zero .or. landon.eq.0) then

        Rsnet= FSWds(i,j)-FSWus(i,j)+FLWds(i,j)-FLWus(i,j)  !net sfc. radiation
                     ! Note: FSWds etc. temporarily stores the averages of 
                     !previous coupling interval
 
        Fsnet= Rsnet-Evap(i,j)-FTs(i,j)   !net energy absorbed by surface

        Tnow(i,j)=Tnow(i,j)+float(intcpl)*86400.*(Fsnet-qfx(i,j))/Cmx
 
c        arr1(i,j)=intcpl*86400.*(Fsnet-qfx(i,j))/Cmx

       end if
      end do
      end do

      return
      end


      Subroutine getSST(T_out)
c************
c  A method for returning the Sea Surface Temperature to other models
c  In this case, it simply provides a pointer to the ocean's data
c  structure.
c
      implicit none
      include 'hgrid.h'
      include 'mxlayer.h'
      REAL T_out(nx,ny)
c************************************************************
 
      integer i,j
 
      do j=1,NY
      do i=1,NX
        T_out(i,j) = Tnow(i,j)
      end do
      end do
 
      return
      end

c************
