C*********************************************************************
c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/output_nc.F,v $
c $Author: munnich $
c $Revision: 1.17 $
c $Date: 2000/08/03 18:27:53 $
c
c
c This file contains the output subroutines for qtcm model
c
c
c User control on output:
c   1. To skip variable for output:
c      Simply comment out the corresponding call to [nc_write_i] in [outpinst]
c      for instantenous values and/or nc_write and [oacc] in [varmean] for mean values 
c      such as 
c         call nc_write_i(u1)  !output instantenous values of u1
c         call nc_write(u1)  !output instantenous values of u1
c         call oacc(u1)        !accumulate values of u1 
c      one can also comment out in bulk the
c      call to [outpinst]/[outpmean] in [outpall] so not to output at all;
c      one can also control at an even higher level in the main program
c      [driver], or by changing the values of <ntouti>,<ntout>(see following)
c   2. As in 1, but add; 
c   3. To control how often and where to output a variable:
c      By default, output is done every <ntout> days to the three files
c      described above, and the first <noout> days are not output at all.  
c      <ntout> can also be specified as a user-
c      control parameter in driver.in (via *.sh), as well as <ntouti>,
c      which controls the instantaneous value output.
c      Note: The minimum value for <ntout> and <ntouti> is 1 day, so if you 
c      need to output
c      more often than a day, you need to call the output routines from with-
c      in the atmosphere-land coupling loop in [qtcm]
c
c Usage note:
c   for output means, make sure to call [varmean] from the innermost loop of
c     the model (in the atmo-land coupling loop in [qtcm]).
c
c The following header files are used using INCLUDE:
c   hgrid.h      Dimensions
c   qtcm.h       Primary subroutine common blocks and parameter of qtcm
c   land.h       
c   calendar.h   Variable declarations for virtual calendar
c   outpmean.h      For output mean values
c
c
c Output the following (see [outpall]):
c   4. output for restart                    in qtcm.f;file=qtcm.restart.out
c*********************************************************************
c

      subroutine outpinit ! dummy for netCDF output
      return
      end
             subroutine outpall
c************
c Output the following:
c 1.  instantaneous value of selected fields    file_i_nc
c 2.  averaged values of selected fields        file_m_nc

      implicit none
#include "driver.h"
#include "calendar.h"
      
      logical outtime
      
c*****************************************************************

c output instanteneous values 
c  if need to call within a day, call from within boundary-atmo coupling 
c   loop in [qtcm]
      if (outtime(ntouti,noout)) call outpi_nc
      
c output mean values
      if (outtime(ntout ,noout)) call outpm_nc

c Output all the prognostic variables for restart run every 12*ntout
c   time step in case the model is abnormally interupted
      if(dayofyear.eq.DAYSPERYEAR) call out_restart !restart at end of year
      if(mod(dayofmodel,12*ntout).eq.0) call out_restart 

      return
      end



             subroutine outpi_nc
c************
c Output instantaneous fields to netcdf file file_nc; 
c
      implicit none
#include "hgrid.h"
#include "qtcm.h"
! for runname:
#include "driver.h"
! WD and Runf
#include "land.h"
! dayofmodel
#include "calendar.h"

      character*80 file_nc
      integer ll,lnblnk
      logical firstcall

      data firstcall /.true./
      save firstcall, file_nc
c
c*****************************************************************

      if(firstcall) then
c create netcdf file and write stype
        file_nc=runname
        call nc_init(file_nc,nx,ny,.false.)      !initialize inst. netcdf file
        call nc_write_stype(file_nc,stype,nx,ny)
        firstcall=.false.
      endif
      ll=lnblnk(file_nc)
      print*,'Outpi_nc: Writing inst. data to "',file_nc(1:ll),'"'
c write explicitly for more control
      call nc_write_i(file_nc,'time    ',u0,dayofmodel)
      call nc_write_i(file_nc,'u0      ',u0, dayofmodel)
      call nc_write_i(file_nc,'u1      ',u1, dayofmodel)
      call nc_write_i(file_nc,'v0      ',v0(1,1), dayofmodel)
      call nc_write_i(file_nc,'v1      ',v1(1,1), dayofmodel)
      call nc_write_i(file_nc,'T1      ',t1, dayofmodel)
      call nc_write_i(file_nc,'q1      ',q1, dayofmodel)
      call nc_write_i(file_nc,'vort0   ',vort0, dayofmodel)
      call nc_write_i(file_nc,'psi0    ',psi0(1,1), dayofmodel)
      call nc_write_i(file_nc,'Ts      ',Ts, dayofmodel)
      call nc_write_i(file_nc,'Prec    ',Qc, dayofmodel)
      call nc_write_i(file_nc,'cl3     ',cl3, dayofmodel)
      call nc_write_i(file_nc,'S0      ',S0, dayofmodel)
      call nc_write_i(file_nc,'FSWds   ',FSWds, dayofmodel)
      call nc_write_i(file_nc,'FSWus   ',FSWus, dayofmodel)
      call nc_write_i(file_nc,'FSWut   ',FSWut, dayofmodel)
      call nc_write_i(file_nc,'FLWds   ',FLWds, dayofmodel)
      call nc_write_i(file_nc,'FLWus   ',FLWus, dayofmodel)
      call nc_write_i(file_nc,'OLR     ',FLWut, dayofmodel)
      call nc_write_i(file_nc,'FTs     ',FTs, dayofmodel)
      call nc_write_i(file_nc,'Evap    ',Evap, dayofmodel)
      call nc_write_i(file_nc,'Runf    ',Runf, dayofmodel)
      call nc_write_i(file_nc,'WD      ',WD, dayofmodel)
      call nc_write_i(file_nc,'taux    ',taux, dayofmodel)
      call nc_write_i(file_nc,'tauy    ',tauy, dayofmodel)
      call nc_write_i(file_nc,'advT1   ',advT1, dayofmodel)
      call nc_write_i(file_nc,'advq1   ',advq1, dayofmodel)
      call nc_write_i(file_nc,'GMq1    ',GMq1, dayofmodel)
      call nc_write_i(file_nc,'Evapi   ',arr1, dayofmodel)
      call nc_write_i(file_nc,'arr2    ',arr2, dayofmodel)
      call nc_write_i(file_nc,'wet     ',arr3, dayofmodel)
      call nc_write_i(file_nc,'Runs    ',arr4, dayofmodel)
!      call nc_write_i(file_nc,'arr1    ',arr1, dayofmodel)
!      call nc_write_i(file_nc,'arr3    ',arr3, dayofmodel)
!      call nc_write_i(file_nc,'arr4    ',arr4, dayofmodel)
      return
      end


             subroutine outpm_nc
c************
c Output pmean in netcdf file file_nc; 
c
      implicit none
! NX, NY
#include "hgrid.h"
! stype
#include "qtcm.h"
! for dayofmodel
#include "calendar.h" 

      character*80 file_nc
      integer ll,lnblnk
      logical firstcall
      
      data firstcall /.true./
      save firstcall, file_nc
c
c*****************************************************************

c write explicitly for more control

      if(firstcall) then
        call nc_init(file_nc,nx,ny,.true.)      !initialize mean netcdf file
        call nc_write_stype(file_nc,stype,nx,ny)
        firstcall=.false.
      endif

      ll=lnblnk(file_nc)
      print*,'Outpm_nc: Writing mean data to  "',file_nc(1:ll),'"'

!-- Caution: strings are case sensitive for nc_write
        call nc_write(file_nc,'time    ',dayofmodel)
        call nc_write(file_nc,'u0      ', dayofmodel)
        call nc_write(file_nc,'u1      ', dayofmodel)
        call nc_write(file_nc,'v0      ', dayofmodel)
        call nc_write(file_nc,'v1      ', dayofmodel)
        call nc_write(file_nc,'T1      ', dayofmodel)
        call nc_write(file_nc,'q1      ', dayofmodel)
        call nc_write(file_nc,'vort0   ', dayofmodel)
        call nc_write(file_nc,'psi0    ', dayofmodel)
        call nc_write(file_nc,'Ts      ', dayofmodel)
        call nc_write(file_nc,'Prec    ', dayofmodel)
        call nc_write(file_nc,'cl3     ', dayofmodel)
        call nc_write(file_nc,'S0      ', dayofmodel)
        call nc_write(file_nc,'FSWds   ', dayofmodel)
        call nc_write(file_nc,'FSWus   ', dayofmodel)
        call nc_write(file_nc,'FSWut   ', dayofmodel)
        call nc_write(file_nc,'FLWds   ', dayofmodel)
        call nc_write(file_nc,'FLWus   ', dayofmodel)
        call nc_write(file_nc,'OLR     ', dayofmodel)
        call nc_write(file_nc,'FTs     ', dayofmodel)
        call nc_write(file_nc,'Evap    ', dayofmodel)
        call nc_write(file_nc,'Runf    ', dayofmodel)
        call nc_write(file_nc,'WD      ', dayofmodel)
        call nc_write(file_nc,'taux    ', dayofmodel)
        call nc_write(file_nc,'tauy    ', dayofmodel)
        call nc_write(file_nc,'advT1   ', dayofmodel)
        call nc_write(file_nc,'advq1   ', dayofmodel)
        call nc_write(file_nc,'GMq1    ', dayofmodel)

        call nc_write(file_nc,'Evapi   ', dayofmodel)
        call nc_write(file_nc,'wet     ', dayofmodel)
        call nc_write(file_nc,'Runs    ', dayofmodel)

!        call nc_write(file_nc,'arr1    ', dayofmodel)
        call nc_write(file_nc,'arr2    ', dayofmodel)
!        call nc_write(file_nc,'arr3    ', dayofmodel)
!        call nc_write(file_nc,'arr4    ', dayofmodel)
      return
      end


             subroutine varmean(it)
c************
c accumulate for the variables that need averaging
c
      implicit none
      include 'hgrid.h'
      include 'qtcm.h'
      include 'land.h'
#include "driver.h"
#include "calendar.h"
      
      integer it,nastep
      
      if(dayofmodel.le.noout) return

      call oacc( u1,'u1')    
      call oacc(v1 ,'v1')    
      call oacc(T1 ,'T1')    
      call oacc(q1 ,'q1')    
      if(mod(it,mt0).eq.0) call oacc(u0 ,'u0')    
      if(mod(it,mt0).eq.0) call oacc(v0 ,'v0')    
      if(mod(it,mt0).eq.0) call oacc(vort0 ,'vort0')    
      if(mod(it,mt0).eq.0) call oacc(psi0 ,'psi0')    
      call oacc(Ts ,'Ts')
      call oacc(Qc ,'Prec')
      call oacc(cl3 ,'cl3')
      if(mod(it,nastep).eq.0) call oacc(S0 ,'S0')    
      call oacc(FSWds ,'FSWds')
      call oacc(FSWus ,'FSWus')
      call oacc(FSWut ,'FSWut')
      call oacc(FLWds ,'FLWds')
      call oacc(FLWus ,'FLWus')
      call oacc(FLWut ,'OLR')
      call oacc(FTs ,'FTs')
      call oacc(Evap ,'Evap')
      call oacc(Runf ,'Runf')
      call oacc(WD ,'WD')
!      if(mod(it,nastep).eq.0) call oacc(STYPE ,'STYPE')    
      call oacc(taux ,'taux')
      call oacc(tauy ,'tauy')
      call oacc(advT1 ,'advT1')
      call oacc(advq1 ,'advq1')
      call oacc(GMq1 ,'GMq1')
      call oacc(arr1 ,'Evapi')
      call oacc(arr2 ,'arr2')
      call oacc(arr3 ,'wet')
      call oacc(arr4 ,'Runs')
!      call oacc(arr1 ,'arr1')
!      call oacc(arr2 ,'arr2')
!      call oacc(arr3 ,'arr3')
!      call oacc(arr4 ,'arr4')

      return
      end


      subroutine oacc(p,var)
c**************
c Accumulate (sum up) for variable p. 
c Like pacc in generic routine  "pavr.f"
c******************************************************************
      implicit none
      include 'hgrid.h'
      include 'outpmean_nc.h'

c arguments
      real p(NXY)
      character*(*) var

c- Local Variables and Functions
      integer i,m
      integer var_index,num_var

      data num_var/0/                   !variable number corres. to
      save num_var


c== identify the variable var and link it to the right place; initializing; 
c 
c Assign a value of 'var_index' corresponding to each var called 
c (like num_var above)

      do m=1,NVAR
         if (var.eq.var0(m)) then
             var_index=m
             go to 33            !Nobody likes 'go to', fix this in the future
                                 !  caution: the indexing is tricky.
         end if
      end do

c this loads up the array var0 with variable names, only first
c time per variable; 
        num_var=num_var+1
        if (num_var.gt.NVAR) then       
           write(*,*) 'Error: capacity of array "var0" exceeded.'
           write(*,*) 'Remedy: increase size of NVAR in outpmean_nc.h'
           stop
        end if
        var0(num_var)=var
        var_index=num_var

33	continue

c== sum up
        npsum(var_index)=npsum(var_index)+1
        call dosum(NXY,p,psum(1,var_index))

      return
      end


      subroutine dosum(NXY,p,psum)
c add array p to psum
      implicit none
      integer NXY
      real p(NXY), psum(NXY)
      psum=psum+p
      return
      end

      subroutine geto(pmean,var)
c
c pass the average in pmean and reset psum and the counter npsum
c 
      implicit none
      include 'hgrid.h'
      include 'outpmean_nc.h'

c Local variables
      integer i,m
      integer index
      real rpsum,pmean(NXY)
      character*(*) var
    
c- find which variable it is
      do m=1,NVAR
         if (var.eq.var0(m)) then
             index=m
         end if
      end do

c- averaging; reset the counter and the array
      call domean(NXY,npsum(index),psum(1,index),pmean)

      return
      end
      

      subroutine domean(nxy,nsum,sum,mean)
!
!-- compute mean from psum of np values
!   and reset counter and array
!
      implicit none
      integer nxy,nsum
      real sum(nxy), mean(nxy)
      real divisor
      divisor=1./nsum
      mean=sum*divisor ! array
      sum=0.
      nsum=0
      return
      end
      

      logical function outtime(ntout,noout)
!************
!     --- decide if the model time is an output time

      implicit none
   
#include "calendar.h"
      integer  ntout,noout

      logical outtime

      outtime=.false.   

      if(dayofmodel.le.noout) return  !no output the first noout days

      if(ntout.eq.-30) then     ! output at end of month after noout days
         outtime=dayofmonth.eq.monlen(monthofyear) 
      else                      ! outpmean every <ntout> days after noout days
         outtime=mod(dayofmodel-noout,ntout).eq.0
      endif

      return
      end
