c*********************
c
c File:  pavr.f 
c
c A 'generic' routine calculating the time average of multiple variables
c
c History:
c   Original version:   N. Zeng at ENEA/Casaccia, Rome      10/98 
c   Modified:           N.Zeng, modifing K.Hales' technique to handle 
c                         multi-variables (similar to bndry.f QTCM2.1)  10/99
c Comment:
c   This is a much more elegant way of handling averaging than in the original 
c     QTCM1V2.1. This can be used for output monthly data, or get mean fluxes
c     for air-sea or land-vegetation coupling.
c
c Usage:
c   call pacc(swet,'swet')          at each time step 
c   call getp(swetm,'swet')         at a specified frequency (e.g, every month)
c 
c     -> then swetm returns the mean of variable swet averaged over the
c        period (in the above example: monthly)
c     Note 1. 'swet' is just an identifier which can be anything as long
c           as it is not already used by another variable.
c          2. the calling subroutine doesn't need to share any 
c             common variables with pavr.f which only needs hgrid.h
c             for the dimension NXY; While NXY can be passed as an argument
c             we chose otherwise so as to make the calling simpler.
c   
c*********************************************************************


      subroutine pacc(p,var)
c**************
c Accumulate (sum up) for variable p
c******************************************************************
      implicit none
      include 'hgrid.h'
      include 'pavr.h'

c arguments
      real p(NXY)
      character*(*) var

c- Local Variables and Functions
      integer i,m
      integer var_index,num_var

      data num_var/0/                   !variable number corres. to
      save num_var


c== identify the variable var and link it to the right place; initializing; 
c 
c Assign a value of 'var_index' corresponding to each var called 
c (like num_var above)
c 'var_index' is what we called 'k'

      do m=1,NVAR
         if (var.eq.var0(m)) then
             var_index=m
             go to 33            !Nobody likes 'go to', fix this in the future
                                 !  caution: the indexing is tricky.
         end if
      end do

c this loads up the array var0 with variable names, only first
c time per variable; 
        num_var=num_var+1
        if (num_var.gt.NVAR) then       
           write(*,*) 'Error: capacity of array "var0" exceeded.'
           write(*,*) 'Remedy: increase size of NVAR in pavr.f'
           stop
        end if
        var0(num_var)=var
        var_index=num_var

c       write(*,*) 'var0,num_var=',var0(num_var),num_var
c===============================================

33	continue

c== sum up
      npsum(var_index)=npsum(var_index)+1
      do i=1,NXY
          psum(i,var_index) = psum(i,var_index) + p(i)
      end do
c===============================================


      return
      end



      subroutine getp(pmean,var)
c
c pass the average in pmean and reset psum and the counter npsum
c 
      implicit none
      include 'hgrid.h'
      include 'pavr.h'

c Local variables
      integer i,m
      integer var_index
      real rpsum,pmean(NXY)
      character*(*) var
    

c- find which variable it is
      do m=1,NVAR
         if (var.eq.var0(m)) then
             var_index=m
         end if
      end do

c     write(*,*)'var_index,var,npsum',var_index,var,npsum(var_index)

c- averaging; reset the counter and the array
      rpsum = 1./npsum(var_index)
      do i=1,NXY
          pmean(i) = psum(i,var_index) * rpsum
          psum(i,var_index) = 0.
      end do
      npsum(var_index) = 0

      
      return
      end
