
      Subroutine TimeInterp(Prior, Upcoming, Result ,t1,t2,now, nx,ny)
c
c  A generic time interpolator.
c
      implicit none
      integer t1,t2
      REAL now
      integer nx,ny
      REAL Prior(nx,ny),Upcoming(nx,ny),Result(nx,ny)
      REAL divisor,fraction
 
      integer i,j
 
      divisor = t2 - t1
      if ( divisor .gt. 0.0 ) then
         fraction = (now - t1)/divisor
c        Result = Prior + fraction*(Upcoming - Prior)  !left from a f90 version
         do j=1,NY
         do i=1,NX
           Result(i,j)=Prior(i,j)+fraction*(Upcoming(i,j)-Prior(i,j))
         end do
         end do
      else
c        Result = Prior
         do j=1,NY
         do i=1,NX
           Result(i,j) = Prior(i,j)
         end do
         end do
      endif
      return
      end
 

             subroutine humtable
c************
c Set up look-up table for saturation humidity every .1K
c between 200K and 400K
c
      implicit none
 
c --Buried Common Variables:
      REAL hTmin,hTmax,hdT,hdTi,hums
      common/hsatcom/ hTmin,hTmax,hdT,hdTi,hums(2001)
 
c --Local Variables & Functions:
      REAL T
      REAL hsat0         ! function
      integer NT
      integer k
c*****************************************************************
 
      hTmin=200.
      hTmax=400.
      hdT=0.1
      hdTi=1./hdT
      NT=(hTmax-hTmin)/hdT+1
      do k=1,NT
          T=hTmin+hdT*(k-1)
          hums(k)=hsat0(T)
      end do
 
      return
      end

             REAL function hsat(T)
c************
c Calculates hsat using a table. Linear interpolation
c used at points in-between.
c
      implicit none
 
c --Arguments:
      REAL T
 
c --Buried Common Variables:
      REAL hTmin,hTmax,hdT,hdTi,hums
      common/hsatcom/ hTmin,hTmax,hdT,hdTi,hums(2001)
 
c --Local Variables & Functions:
      REAL w
      integer k
c*****************************************************************
 
      if(T.lt.hTmin) then
c       write(*,*) 'terror !!!! T below 200K, hsat has been set to 200K'
       T=hTmin
      end if
      if(T.gt.hTmax) then
c       write(*,*) 'terror !!!! T above 400, hsat has been set to 400K'
       T=hTmax-hdT
      end if
      w=1.0+(T-hTmin)*hdTi
      k=w
      w=w-k
      hsat=(1.0-w)*hums(k)+w*hums(k+1)
 
      return
      end

             subroutine t1ctable
c************
c Set up look-up table for nonlinear T1c
c T1c between -300 and 300 at the interval of 1.
c
      implicit none
 
c --Buried Common Variables:
      REAL T1cs(601),rhsx(601)
      common/T1cscom/ T1cs,rhsx
 
c --Local Variables & Functions:
      integer NP
      REAL prs(11),alpha(11),Tcref(11),a1(11)  ! 11 vertical layers
      REAL qcp(11),qcphat,a1hat,hsat,hlatent,Cpi
 
      integer i,k
c*****************************************************************
      open(31,file='vtpar.in',form='formatted',status='old')
      NP=11
      do k=1,NP
         read(31,*) prs(k),alpha(k),Tcref(k),a1(k)
      enddo
         read(31,*) a1hat
      close(31)
   
      hlatent=2.43d6
      Cpi=1./1.004d3
 
      do i=1,601
         T1cs(i)=-300.+(i-1)*1.
         do k=1,NP
            qcp(k)=alpha(k)*hsat(Tcref(k)+a1(k)*T1cs(i))*Hlatent*Cpi
         enddo
         qcphat=0.0
         do k=1,NP-1
            qcphat=qcphat+(qcp(k)+qcp(k+1))*0.5*(prs(k)-prs(k+1))
         enddo
         qcphat=qcphat/(prs(1)-prs(NP))
         rhsx(i)=a1hat*T1cs(i)+qcphat
      enddo
      return
      end

             REAL function nlt1c(i,j,x)
c************
c Calculates nonlinear T1c using a table. Linear interpolation
c used at points in-between.
c
      implicit none
#include "hgrid.h"
 
c --Arguments:
      integer i,j
      REAL x 
 
c --Buried Common Variables:
      REAL T1cs(601),rhsx(601),alpha
      common/T1cscom/ T1cs,rhsx
 
c --Local Variables & Functions:
      integer k , bisect
      integer k0(NX,NY)   ! guess of neighboring
      data k0 /NXY*0/
      save k0  
c*****************************************************************

      if(x.lt.rhsx(1)) then
       write(*,*) 'terror !!!! T1c < -300. extropolate T1c from -300.'
       write(*,*) 'x=',x
       nlt1c=T1cs(1)+(T1cs(2)-T1cs(1))/(rhsx(2)-rhsx(1))*
     &       (x-rhsx(1)) 
       write(*,*) 'T1cs=',nlt1c
       stop
      end if
      if(x.gt.rhsx(601)) then
       write(*,*) 'terror !!!! T1c > 300. extropolate T1c from 300.'
       write(*,*) 'x=',x
       nlt1c=T1cs(601)+(T1cs(601)-T1cs(600))/(rhsx(601)-rhsx(600))*
     &       (x-rhsx(601)) 
       write(*,*) 'T1cs=',nlt1c
       stop
      end if

      call embrace(rhsx,601,x,k0(i,j))

      k=k0(i,j)
      alpha=(x-rhsx(k))/(rhsx(k+1)-rhsx(k))
      nlt1c=T1cs(k+1)*alpha+T1cs(k)*(1.0-alpha)

      return
      end


      subroutine embrace(xx,n,x,k0)
c************
c  search for embracing k0 with xx(k0)< x < xx(k0+1)
c  k0 on input as initial guess. 
c  Based on Num. Rec. "hunt" routine,
c  "xx" must be increasing.
c
      implicit none
c --Arguments:
      integer n  ! dimension of xx
      integer k0 ! Input: Initial guess,
                 ! Output lower embracing index
      REAL xx(n) ! lookup table
      REAL x     ! value to embrace

c --Local Variables & Functions:
      integer k1,k2,inc,bisect
c*****************************************************************

      if(k0.le.0) then ! no initial guess -> do bisection search
         k0=bisect(xx,n,x) 
         return
      else if (xx(k0).le. x) then !search upward
         inc=1
         k1=k0+inc
         do while (xx(k1).le. x) 
            k0=k1
            inc=2*inc
            k1=min(k1+inc,n)
         enddo 
      else if (xx(k0).gt.x) then !search downward
         inc=1
         k1=k0
         k0=k1-inc
         do while (xx(k0).gt.x)
            inc=2*inc
            k1=k0
            k0=max(k0-inc,1)
         enddo
      endif
      do while (k1-k0.gt.1) !bisection search
        k2=(k1+k0)/2
        if(x.gt.xx(k2)) then
          k0=k2
        else
          k1=k2
        endif
      enddo
      return
      end


      integer function bisect(xx,n,x)
c************
c
c  bisection search for embracing 
c  xx(bisect) < x < xx(bisect+1)
c
      implicit none
      REAL xx(n),x
      integer bisect, n, k1, k2
c*****************************************************************
      bisect=0
      k1=n+1
      do while (k1-bisect.gt.1)
        k2=(k1+bisect)/2
        if(x.gt.xx(k2)) then
          bisect=k2
        else
          k1=k2
        endif
      enddo
      return
      end


             REAL function hsat0(T1)
c************
c Saturation specific humidity at 1000 mb; empirical formula from
c  J. Shuttleworth applicable to normal range of temperature.
c
      implicit none
 
c --Arguments:
      REAL T1      !input T1 in Kelvin
 
c --Local Variables & Functions:
      REAL T,esat
c*****************************************************************
 
      T=T1-273.15                           ! convert to Celsius
      esat=.6108*exp(17.270*T/(237.3+T))  ! unit: kpa
      hsat0=.622*esat*0.01                  ! unit= kg/kg, assuming
c                                           ! Pressure=100kpa
 
      return
      end
 
             REAL function hsats(T)
c************
c Slope of saturation humidity; unit of hsat/K
      implicit none
 
c --Arguments:
      REAL T      !input T in Kelvin

c --local
      REAL hsat
c*****************************************************************
 
        hsats = 10.0*(hsat(T+.050)-hsat(T-.050))
 
      return
      end
 
             REAL function zbar(a,n)

c************
c Mean of an array of size n
c
      implicit none
 
c --Arguments:
c     zbar          ! function:  mean of array a(n)
      integer n     ! array dimension
      REAL a(n)   ! array.  not altered on exit.
 
c --Local Variables & Functions:
      integer i
c*****************************************************************
 
      zbar=a(1)
      do i=2,n
          zbar=zbar+a(i)
      end do
      zbar=zbar/n
 
      return
      end
 
 
             REAL function zbar2(a,b,n)
c************
c Mean of arrays a*b, each of size n
c
        implicit none
 
c --Arguments:
        integer n
        REAL a(n),b(n)
 
c --Local Variables & Functions:
        integer i
c*****************************************************************
 
 
        zbar2=a(1)*b(1)
        do i=2,n
            zbar2=zbar2+a(i)*b(i)
        end do
        zbar2=zbar2/n
 
        return
        end
 
 
 
             subroutine as(a,n,val)
c************
c Assign a single value val to array a of length n; no need if FORTRAN90
c
      implicit none
 
c --Arguments:
      integer n
      REAL a(n),val
 
c --Local Variables & Functions:
      integer i
c*****************************************************************
 
      do i=1,n
        a(i)=val
      end do
 
      return
      end
 
c************************************************************

      integer function lnblnk(c)
      character*(*) c
      integer i
      lnblnk=len(c)
      do i=lnblnk,1,-1
         if(c(i:i).ne.' ') then 
            lnblnk=i
            return
         endif
      enddo
      lnblnk=0
      end
