#!/bin/csh -f
# qtcm_seasonal.sh   NZ 5/1998
# a run with climatological SST (seasonal)


# The solar forcing is set to day 15 of MONTH0

set MONTH0 = 01 

# Set the QTCM root directory (e.g., where your files are untared)
set QTCMROOT = ..       # this is to assume you are in QTCMROOT/work right now

# Set the source directory which contains all the source files
set srcdir = $QTCMROOT/src

set bnddir = $QTCMROOT/bnddata

set workdir = $QTCMROOT/work     # Set the working directory

#- Compile qtcm using make in srcdir, but run it in workdir
cd $srcdir
make qtcm
cd $workdir

# copy qtcm to work directory
cp -fp ../src/qtcm qtcm

# assign the perpetual sst used; the file name has to be 00000000.sst
#   Note: solar at TOA in[solartop] is still varing seasonally, and this
#     should impact land region mostly, also oceanic regions somewhat 
#     through solar absorption
cd $bnddir
cp SST_Reynolds_oi/0000${MONTH0}15.sst SST_Reynolds_oi/00000000.sst  
cd $workdir
#
#Generate the user control parameters.
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='QTCM seasonal  run'      !title
 bnddir='$QTCMROOT/bnddata'      !bnd. data other than SST
 SSTdir='$QTCMROOT/bnddata/SST_Reynolds_oi'    !where SST files are
 outdir='$QTCMROOT/proc'      !where output go
 landon=1                     !if not 1: land is like ocean with fake 'SST'
 SSTmode='perpetual'           !decide what kind of SST to use
 year0 = 1001                 !starting year; must be between 1001 and 9998
 month0 = 1                   !starting month; 1 to 12
 day0 = 15                    !1 t0 31; each month has variable or fixed length
 lastday = 420                !model runs from day 1 to lastday 
 interval = 1                 !ocean-atmo. coupling interval; integer days
 noout=48                     !no output for the first noout days
 ntout=31                     !output mean every ntout days 
 ntouti=31                    !output inst. value every ntouti days 
 mrestart=1                   !=1: restart using qtcm.resart
 dt=1200.                     !atmospheric model time step, real seconds
 &end
EOF

#- Run qtcm using nice; timing the run; diagnostic output to log
/bin/time nice qtcm |& tee log 

