#!/bin/csh -f
# qtcm_seasonal.sh   NZ 5/1998
# a run with climatological SST (seasonal)

# Set the QTCM root directory (e.g., where your files are untared)
set QTCMROOT = ..       # this is to assume you are in QTCMROOT/work right now

# Set the source directory which contains all the source files
set srcdir = $QTCMROOT/src

set workdir = $QTCMROOT/work     # Set the working directory

set thedate = `date +%H%M_%m%d`  # current date and time (used for file names)
set theoutdir = $QTCMROOT/proc # output directory

#- Compile qtcm using make in srcdir, but run it in workdir
pushd $srcdir
make qtcm
popd

# copy qtcm to work directory
cp -fp ../src/qtcm qtcm

#
#Generate the user control parameters.
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='QTCM1V2.2  seasonal  run'
 bnddir='$QTCMROOT/bnddata'      !bnd. data other than SST
 SSTdir='$QTCMROOT/bnddata/SST_Reynolds_oi'    !where SST files are
 outdir='$theoutdir'      !where output go
 runname='$thedate'           ! used for output file names
 landon=1                     !if not 1: land is like ocean with fake 'SST'
 SSTmode='seasonal'           !decide what kind of SST to use
 year0 = 1                    !starting year; must be between 1 and 9998
 month0 = 11                  !starting month; 1 to 12
 day0 = 1                     !1 t0 31; each month has variable or fixed length
 lastday = 1156               !model runs from day 1 to lastday 
 noout=61                     !no output for the first noout days
 ntout=-30      !output mean every ntout days (-30: calendar monthly mean)
 ntouti=-30     !output inst. value every ntouti days (-30 every calendar month)
 mrestart=1                   !=1: restart using qtcm.resart
 dt=1200.                     !atmospheric model time step, real seconds
 &end
EOF

#- Run qtcm using nice; timing the run; diagnostic output to log
/bin/time nice qtcm |& tee log 

